import { ApolloClient, ApolloQueryResult } from "@apollo/client";
import { Link, MinilinksResult } from "./minilinks";
export interface DeepClientOptions<L = Link<number>> {
    apolloClient: ApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
}
export interface DeepClientResult<R> extends ApolloQueryResult<R> {
}
export declare type DeepClientPackageSelector = string;
export declare type DeepClientPackageContain = string;
export declare type DeepClientLinkId = number;
export declare type DeepClientStartItem = DeepClientPackageSelector | DeepClientLinkId;
export declare type DeepClientPathItem = DeepClientPackageContain;
export interface DeepClientInstance<L = Link<number>> {
    apolloClient?: ApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
    select<LL = L>(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<LL[]>>;
    insert<LL = L>(objects: Partial<LL> | Partial<LL>[], options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    update(exp: any, value: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    delete(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        returning: {
            id: any;
        }[];
    }>>;
    reserve<LL = L>(count: number): Promise<number[]>;
    await(id: number): Promise<boolean>;
    id(start: DeepClientStartItem, ...path: DeepClientPathItem[]): Promise<number>;
}
export declare class DeepClient<L = Link<number>> implements DeepClientInstance<L> {
    apolloClient?: ApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
    constructor(options: DeepClientOptions<L>);
    select<LL = L>(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<LL[]>>;
    insert<LL = L>(objects?: Partial<LL> | Partial<LL>[], options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    update(exp: any, value: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    delete(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        returning: {
            id: any;
        }[];
    }>>;
    reserve<LL = L>(count: number): Promise<number[]>;
    await(id: number): Promise<boolean>;
    idFieldNames: string[];
    valueFieldName: string;
    valueRelationNames: {
        string: string;
        number: string;
    };
    boolExpSerialize(exp: any): any;
    id(start: DeepClientStartItem, ...path: DeepClientPathItem[]): Promise<number>;
}
