"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ColumnTypeToSQLColumnType = exports.api = void 0;
const debug_1 = __importDefault(require("debug"));
const client_1 = require("@deep-foundation/hasura/client");
const api_1 = require("@deep-foundation/hasura/api");
const sql_1 = require("@deep-foundation/hasura/sql");
const apollo_boost_1 = require("apollo-boost");
const vm_1 = __importDefault(require("vm"));
const permission_1 = require("../permission");
const global_ids_1 = require("../global-ids");
const promise_1 = require("../promise");
const SCHEMA = 'public';
const debug = (0, debug_1.default)('deepcase:eh');
exports.api = new api_1.HasuraApi({
    path: process.env.MIGRATIONS_HASURA_PATH,
    ssl: !!+process.env.MIGRATIONS_HASURA_SSL,
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const client = (0, client_1.generateApolloClient)({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.MIGRATIONS_HASURA_SSL,
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
exports.ColumnTypeToSQLColumnType = {
    [global_ids_1.GLOBAL_ID_STRING]: 'TEXT',
    [global_ids_1.GLOBAL_ID_NUMBER]: 'float8',
    [global_ids_1.GLOBAL_ID_JSON]: 'jsonb',
};
exports.default = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
    try {
        const event = (_a = req === null || req === void 0 ? void 0 : req.body) === null || _a === void 0 ? void 0 : _a.event;
        const operation = event === null || event === void 0 ? void 0 : event.op;
        if (operation === 'INSERT' || operation === 'UPDATE' || operation === 'DELETE') {
            const oldRow = (_b = event === null || event === void 0 ? void 0 : event.data) === null || _b === void 0 ? void 0 : _b.old;
            const newRow = (_c = event === null || event === void 0 ? void 0 : event.data) === null || _c === void 0 ? void 0 : _c.new;
            const current = operation === 'DELETE' ? oldRow : newRow;
            const typeId = current.type_id;
            console.log('current', current, typeId);
            try {
                const handleStringResult = yield client.query({ query: (0, apollo_boost_1.gql) `query SELECT_CODE($typeId: bigint) { links(where: {
          type_id: { _eq: ${global_ids_1.GLOBAL_ID_SYNC_TEXT_FILE} },
          # to_id: { _eq: 16 },
          # from_id: { _eq:  }
          in: {
            from_id: { _eq: ${global_ids_1.GLOBAL_ID_JS_EXECUTION_PROVIDER} },
            type_id: { _eq: ${global_ids_1.GLOBAL_ID_HANDLER} },
            in: {
              from: {
                type_id: { _eq: $typeId },
              },
              type_id: { _eq: ${global_ids_1.GLOBAL_ID_HANDLE_INSERT} },
            }
          }
        }) {
          id
          value
        } }`, variables: {
                        typeId,
                    } });
                const code = (_g = (_f = (_e = (_d = handleStringResult === null || handleStringResult === void 0 ? void 0 : handleStringResult.data) === null || _d === void 0 ? void 0 : _d.links) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.value) === null || _g === void 0 ? void 0 : _g.value;
                if (code) {
                    try {
                        vm_1.default.runInNewContext(code, { console, Error, oldRow, newRow });
                    }
                    catch (error) {
                        debug('error', error);
                    }
                }
                if (typeId === global_ids_1.GLOBAL_ID_TABLE_VALUE) {
                    const results = yield client.query({ query: (0, apollo_boost_1.gql) `query SELECT_TABLE_STRUCTURE($tableId: bigint) {
            links(where: {id: {_eq: $tableId}}) {
              id
              values: out_aggregate(where: {type_id: {_eq: ${global_ids_1.GLOBAL_ID_TABLE_VALUE}}}) {
                aggregate {
                  count
                }
              }
              columns: out(where: {type_id: {_eq: ${global_ids_1.GLOBAL_ID_TABLE_COLUMN}}}) {
                id column_id: to_id value
              }
            }
          }`, variables: {
                            tableId: current.from_id,
                        } });
                    const table = (_j = (_h = results === null || results === void 0 ? void 0 : results.data) === null || _h === void 0 ? void 0 : _h.links) === null || _j === void 0 ? void 0 : _j[0];
                    const valuesCount = (_l = (_k = table === null || table === void 0 ? void 0 : table.values) === null || _k === void 0 ? void 0 : _k.aggregate) === null || _l === void 0 ? void 0 : _l.count;
                    const tableName = 'table' + current.to_id;
                    const columns = ((table === null || table === void 0 ? void 0 : table.columns) || []).map(c => { var _a; return ({ name: `${((_a = c === null || c === void 0 ? void 0 : c.value) === null || _a === void 0 ? void 0 : _a.value) || 'value'}`, type: (exports.ColumnTypeToSQLColumnType === null || exports.ColumnTypeToSQLColumnType === void 0 ? void 0 : exports.ColumnTypeToSQLColumnType[c === null || c === void 0 ? void 0 : c.to_id]) || 'TEXT' }); });
                    if (['INSERT', 'UPDATE'].includes(operation) && valuesCount === 1) {
                        const createTable = yield exports.api.sql((0, sql_1.sql) `
              CREATE TABLE ${SCHEMA}."${tableName}" (id bigint PRIMARY KEY, link_id bigint NOT NULL, ${columns.map(c => `${c.name} ${c.type}`).join(',')});
              CREATE SEQUENCE ${tableName}_id_seq
              AS bigint START WITH 1 INCREMENT BY 1 NO MINVALUE NO MAXVALUE CACHE 1;
              ALTER SEQUENCE ${tableName}_id_seq OWNED BY ${SCHEMA}."${tableName}".id;
              ALTER TABLE ONLY ${SCHEMA}."${tableName}" ALTER COLUMN id SET DEFAULT nextval('${tableName}_id_seq'::regclass);
            `);
                        const error = (_o = (_m = createTable === null || createTable === void 0 ? void 0 : createTable.data) === null || _m === void 0 ? void 0 : _m.internal) === null || _o === void 0 ? void 0 : _o.error;
                        if (error)
                            console.log(error);
                        yield exports.api.sql((0, sql_1.sql) `
              INSERT INTO "links__tables" (name) VALUES ('${tableName}');
            `);
                        yield exports.api.query({
                            type: 'track_table',
                            args: {
                                schema: SCHEMA,
                                name: tableName,
                            },
                        });
                        yield exports.api.query({
                            type: 'create_object_relationship',
                            args: {
                                table: tableName,
                                name: 'link',
                                using: {
                                    manual_configuration: {
                                        remote_table: {
                                            schema: SCHEMA,
                                            name: 'links',
                                        },
                                        column_mapping: {
                                            link_id: 'id',
                                        },
                                    },
                                },
                            },
                        });
                        yield exports.api.query({
                            type: 'create_array_relationship',
                            args: {
                                table: 'links',
                                name: tableName,
                                using: {
                                    manual_configuration: {
                                        remote_table: {
                                            schema: SCHEMA,
                                            name: tableName,
                                        },
                                        column_mapping: {
                                            id: 'link_id',
                                        },
                                    },
                                },
                            },
                        });
                        yield (0, permission_1.permissions)(exports.api, tableName, {
                            select: {},
                            insert: {},
                            update: {},
                            delete: {},
                        });
                    }
                    else if (operation === 'DELETE' && !valuesCount) {
                        yield exports.api.sql((0, sql_1.sql) `
              DELETE FROM "links__tables" WHERE name='${tableName}';
            `);
                        yield exports.api.query({
                            type: 'untrack_table',
                            args: {
                                table: {
                                    schema: SCHEMA,
                                    name: tableName,
                                },
                                cascade: true,
                            },
                        });
                        yield exports.api.sql((0, sql_1.sql) `
              DROP TABLE "${tableName}" CASCADE;
            `);
                    }
                }
                if (operation === 'INSERT' && !global_ids_1.DENIED_IDS.includes(current.type_id) && global_ids_1.ALLOWED_IDS.includes(current.type_id)) {
                    debug('resolve', current.id);
                    yield (0, promise_1.resolve)({ id: current.id, client });
                }
                return res.status(200).json({});
            }
            catch (error) {
                debug('error', error);
                if (operation === 'INSERT' && !global_ids_1.DENIED_IDS.includes(current.type_id) && global_ids_1.ALLOWED_IDS.includes(current.type_id)) {
                    debug('reject', current.id);
                    yield (0, promise_1.reject)({ id: current.id, client });
                }
            }
            return res.status(500).json({ error: 'notexplained' });
        }
        return res.status(500).json({ error: 'operation can be only INSERT or UPDATE' });
    }
    catch (error) {
        return res.status(500).json({ error: error.toString() });
    }
});
//# sourceMappingURL=links.js.map